package com.zehong.system.service.impl.tradeSettlement;

import com.zehong.common.core.exception.BusinessException;
import com.zehong.common.utils.SecurityUtils;
import com.zehong.system.domain.TAccount;
import com.zehong.system.domain.TCashOperate;
import com.zehong.system.domain.TTradeProject;
import com.zehong.system.mapper.TAccountMapper;
import com.zehong.system.mapper.TCashOperateMapper;
import com.zehong.system.mapper.TTradeProjectMapper;
import io.jsonwebtoken.lang.Collections;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @author geng
 * 交易结算
 */
@Component
public class SettlePendingPayment {

    private static final Logger log = LoggerFactory.getLogger(SettlePendingPayment.class);

    private TTradeProject tradeProject;

    @Resource
    private TAccountMapper tAccountMapper;

    @Resource
    private TTradeProjectMapper tTradeProjectMapper;

    @Resource
    private TCashOperateMapper cashOperateMapper;

    @Autowired
    private AutoSettlePendingPayment autoSettlePendingPayment;

    /**
     * 可用账户结算
     */
    public void settlePendingPayment(TTradeProject tradeProject){
        this.tradeProject = tradeProject;
        //买方结算
        BigDecimal payment = settleBuyerAbleAmount();
        //卖方结算
        settleSellerAbleAmount(payment.abs());
    }


    /**
     * 买方结算
     */
    private BigDecimal settleBuyerAbleAmount() {
        TAccount account = getAccountInfo(this.tradeProject.getTradeDeptId());
        //判断买方可用额度是否为0
        if(account.getAbleAmount().compareTo(BigDecimal.ZERO) == 0){
            throw new BusinessException("账户金额不足");
        }
        //买方可用额度为0，计算尾款
        BigDecimal pendingPayment = account.getAbleAmount().compareTo(tradeProject.getPendingPayment()) == 1 ?
                new BigDecimal("0") :
                this.tradeProject.getPendingPayment().subtract(account.getAbleAmount());
        //买方可用额度计算
        BigDecimal payment = pendingPayment.compareTo(BigDecimal.ZERO) == 0 ?
                this.tradeProject.getPendingPayment().negate() :
                account.getAbleAmount().negate();
        this.tradeProject.setPendingPayment(pendingPayment);
        updateAccount(account,payment);
        return payment;
    }

    /**
     * 卖方结算
     */
    private void settleSellerAbleAmount(BigDecimal incomeAmount) {
        TAccount account = getAccountInfo(this.tradeProject.getApplyDeptId());
        //更新可用额度
        updateAccount(account,incomeAmount);
        autoSettlePendingPayment.autoSettlePendingPayment(incomeAmount,account,this.tradeProject.getApplyDeptId());
    }

    /**
     * 获取账户信息
     * @param deptId
     * @return
     */
    private TAccount getAccountInfo(Long deptId){
        TAccount tAccount = new TAccount();
        tAccount.setDeptId(deptId);
        List<TAccount> accounts =  tAccountMapper.selectTAccountList(tAccount);
        if(Collections.isEmpty(accounts)){
            log.error("交易部门资金账户不能为空，部门id:" + tradeProject.getApplyDeptId());
            throw new BusinessException("交易部门资金账户不能为空!");
        }
        return accounts.get(0);
    }


    /**
     * 更新账户可用金额
     * @param tAccount 账户信息
     * @param  amount 账户变动金额
     */
    private void updateAccount(TAccount tAccount,BigDecimal amount){
        if(amount.compareTo(BigDecimal.ZERO) == 0) return;
        tAccount.setAbleAmount(tAccount.getAbleAmount().add(amount));
        tAccount.setUpdateTime(new Date());
        tAccountMapper.updateTAccount(tAccount);
        //更新资金变动日志
        TCashOperate operate = new TCashOperate();
        operate.setOperateDeptId(tAccount.getDeptId());
        operate.setOperateAmount(amount.compareTo(BigDecimal.ZERO) == 1?amount:amount.abs());
        operate.setOperateTime(new Date());
        operate.setRelationDoc(this.tradeProject.getTradeId().toString());
        //1.交易单 2.借支单 3.外部采购单 4.借贷单
        operate.setDocumentType("1");
        operate.setOperateType(amount.compareTo(BigDecimal.ZERO) == 1 ? "1" : "2");
        operate.setOperatePersonId(SecurityUtils.getLoginUser().getUser().getUserId());
        cashOperateMapper.insertTCashOperate(operate);
    }

}
